#include <fmt/format.h>

#include "debug_messenger.hpp"

fn DebugMessenger::create(const vk::Instance& instance) -> vk::UniqueDebugUtilsMessengerEXT {
#ifdef NDEBUG
  return nullptr;
#else
  vk::DebugUtilsMessengerCreateInfoEXT createInfo {
    .messageSeverity = vk::DebugUtilsMessageSeverityFlagBitsEXT::eVerbose |
                       vk::DebugUtilsMessageSeverityFlagBitsEXT::eWarning |
                       vk::DebugUtilsMessageSeverityFlagBitsEXT::eError,
    .messageType = vk::DebugUtilsMessageTypeFlagBitsEXT::eGeneral |
                   vk::DebugUtilsMessageTypeFlagBitsEXT::eValidation |
                   vk::DebugUtilsMessageTypeFlagBitsEXT::ePerformance,
    .pfnUserCallback = debugCallback,
    .pUserData       = nullptr,
  };

  return instance.createDebugUtilsMessengerEXTUnique(createInfo);
#endif
}

VKAPI_ATTR VkBool32 VKAPI_CALL DebugMessenger::debugCallback(
  VkDebugUtilsMessageSeverityFlagBitsEXT /*messageSeverity*/,
  VkDebugUtilsMessageTypeFlagsEXT /*messageType*/,
  const VkDebugUtilsMessengerCallbackDataEXT* pCallbackData,
  void* /*pUserData*/
) {
  fmt::println("validation layer: {}", pCallbackData->pMessage);
  return VK_FALSE;
}
