#pragma once

#include <optional>
#include "../util/types.hpp"

/**
 * @brief Struct to store queue family indices.
 *
 * This struct contains the indices of the graphics and presentation queue families.
 */
struct QueueFamilyIndices {
    std::optional<u32> graphics_family; ///< Index of graphics queue family
    std::optional<u32> present_family;  ///< Index of presentation queue family

    /**
     * @brief Check if all required queue families are found.
     *
     * @return True if both graphics and presentation families are found, false otherwise.
     */
    [[nodiscard]] fn isComplete() const -> bool { 
        return graphics_family.has_value() && present_family.has_value(); 
    }
};
